/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at51decision.c	general decisions
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include "at51decision.h"

int at51_dec_check_command_compillance(at51_param_t *at51_param) {
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_WRITE))
	return -1;
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_BURN))
	return -1;
    if ((at51_param->command & _CMD_WRITE) &&
	(at51_param->command & _CMD_BURN))
	return -1;
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_ERASE))
	return -1;
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_LOCK1))
	return -1;
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_LOCK2))
	return -1;
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_LOCK3))
	return -1;
    if ((at51_param->command & _CMD_READ) &&
	(at51_param->command & _CMD_LOCK4))
	return -1;
    if ((at51_param->command & _CMD_BURN) &&
	(at51_param->command & _CMD_LOCK1))
	return -1;
    if ((at51_param->command & _CMD_BURN) &&
	(at51_param->command & _CMD_LOCK2))
	return -1;
    if ((at51_param->command & _CMD_BURN) &&
	(at51_param->command & _CMD_LOCK3))
	return -1;
    if ((at51_param->command & _CMD_BURN) &&
	(at51_param->command & _CMD_LOCK4))
	return -1;
    return 0;
} /* end of at51_dec_check_command_compillance() */

int at51_dec_lookup_partindex(at51_param_t *at51_param) {
    int Li = 0;
    
    if (strlen(at51_param->partname) == 0) {
	at51_param->partindex = -1;
	return -1;
    }
    while (strlen(at51_db[Li].partname)) {
	if (strcmp(at51_db[Li].partname, at51_param->partname) == 0) {
	    at51_param->partindex = Li;
	    return 0;
	}
	Li++;
    }
    at51_param->partindex = -1;
    return -1;
} /* end of at51_dec_lookup_partindex() */

int at51_dec_process_device_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;

    if (strlen(at51_param->partname) == 0) {
	/* Nothing to do - no specified device */
	printf("Error: Nothing to do - no specified device\n");
	*dec_error = EINVAL;
	*dec_status = _S_DEC_ERRORNOSPECIFIEDDEVICE;
	return -1;
    }
    if (at51_param->partindex == -1) {
	/* Unsupoorted device */
	printf("Unsupoorted device\n");
	*dec_error = EINVAL;
	*dec_status = _S_DEC_ERRORNOSPECIFIEDDEVICE;
	return -1;
    }
    if (at51_param->command == 0) {
	/* Nothing to do - no specified command */
	printf("Nothing to do - no specified command\n");
	*dec_error = EINVAL;
	*dec_status = _S_DEC_ERRORNOSPECIFIEDCOMMAND;
	return -1;
    }
    if (strcmp(at51_param->partname,"AT89C1051") == 0) {
	Lres = at51_dec_process_device_AT89C1051_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89C2051") == 0) {
	Lres = at51_dec_process_device_AT89C2051_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89C4051") == 0) {
	Lres = at51_dec_process_device_AT89C4051_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89C51") == 0) {
	Lres = at51_dec_process_device_AT89C51_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89C52") == 0) {
	Lres = at51_dec_process_device_AT89C52_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89S51") == 0) {
	Lres = at51_dec_process_device_AT89S51_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89S52") == 0) {
	Lres = at51_dec_process_device_AT89S52_command(at51_param,
	    dec_error, dec_status);
	return Lres;
    }
    if (strcmp(at51_param->partname,"AT89C55WD") == 0) {
	printf("%s Under construction!\n", at51_param->partname);
	*dec_error = EINVAL;
	*dec_status = _S_DEC_ERRORUNDERCONSTRUCTION;
	return -1;
    }
    return 0;
} /* end of at51_dec_process_device_command() */

int at51_dec_process_device_AT89S51_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 4096;

    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89s51_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89S51 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Readlock... ");
	    fflush(stdout);
	    Lres = at51_at89s51_readlock(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device readlock operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    if (Lpro_status == _S_PRO_OKDEVICELOCKED) printf("but continue with reading...\n");
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89s51_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
	        return -1;
	    }
	}
	else {
	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89S51 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89s51_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Erasing... not exist for this device\n");
	}
    } /* end of erasing AT89S51 */
    if (at51_param->command & _CMD_WRITE) { 
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("error - see below!\n");
		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89s51_write(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    		case _S_PRO_ERRORINVALIDREQUEST:
	    		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
			case _S_PRO_ERRORWRITINGDEVICE:
			    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
			default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
		    }
		    return -1;
		}
	    }
	}
	else {
	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89S51 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
    	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
    	    if (Lres == -1) {
    		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89s51_burn(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
	    printf("Burning... not exist for this device\n");
	}
    } /* end of burning AT89S51 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
    	    printf("Locking 1... ");
    	    fflush(stdout);
	    Lres = at51_at89s51_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89S51 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89s51_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89S51 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
	    printf("Locking 3... ");
    	    fflush(stdout);
	    Lres = at51_at89s51_lock3(at51_param, &Lpro_error, &Lpro_status);
    	    if (Lres == -1) {
    		printf("Error: device lock 3 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89S51 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89S51 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	    printf("Readlock... ");
	    fflush(stdout);
	    Lres = at51_at89s51_readlock(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device readlock operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
    	    }
	}
	else {
    	    printf("Reading Lock... not exist for this device\n");
	}
    } /* end of reading lock bits AT89S51 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89S51_command() */

int at51_dec_process_device_AT89S52_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 8192;

    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89s52_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89S52 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Readlock... ");
	    fflush(stdout);
	    Lres = at51_at89s52_readlock(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device readlock operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    if (Lpro_status == _S_PRO_OKDEVICELOCKED) printf("but continue with reading...\n");
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89s52_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	}
	else {
    	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89S52 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89s52_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Erasing... not exist for this device\n");
	}
    } /* end of erasing AT89S52 */
    if (at51_param->command & _CMD_WRITE) {
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("error - see below!\n");
		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89s52_write(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    		case _S_PRO_ERRORINVALIDREQUEST:
	    		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
			case _S_PRO_ERRORWRITINGDEVICE:
			    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
			default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
		    }
		    return -1;
		}
	    }
	}
	else {
    	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89S52 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
    	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
    	    if (Lres == -1) {
    		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89s52_burn(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
    	    printf("Burning... not exist for this device\n");
	}
    } /* end of burning AT89S52 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
    	    printf("Locking 1... ");
    	    fflush(stdout);
	    Lres = at51_at89s52_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89S52 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89s52_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89S52 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
    	    printf("Locking 3... ");
    	    fflush(stdout);
    	    Lres = at51_at89s52_lock3(at51_param, &Lpro_error, &Lpro_status);
    	    if (Lres == -1) {
    		printf("Error: device lock 3 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89S52 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89S52 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	    printf("Readlock... ");
	    fflush(stdout);
	    Lres = at51_at89s52_readlock(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device readlock operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Reading Lock... not exist for this device\n");
	}
    } /* end of reading lock bits AT89S52 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89S52_command() */

int at51_dec_process_device_AT89C51_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 4096;
    
    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89c51_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89C51 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89c51_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	}
	else {
    	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89C51 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89c51_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Erasing Lock... not exist for this device\n");
	}
    } /* end of erasing AT89C51 */
    if (at51_param->command & _CMD_WRITE) {
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("error - see below!\n");
		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c51_write(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    		case _S_PRO_ERRORINVALIDREQUEST:
	    		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
			case _S_PRO_ERRORWRITINGDEVICE:
			    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
			default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
		    }
		    return -1;
		}
	    }
	}
	else {
    	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89C51 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
    	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
    	    if (Lres == -1) {
    		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c51_burn(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
    	    printf("Burning Lock... not exist for this device\n");
	}
    } /* end of burning AT89C51 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
    	    printf("Locking 1... ");
    	    fflush(stdout);
	    Lres = at51_at89c51_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89C51 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89c51_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89C51 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
    	    printf("Locking 3... ");
    	    fflush(stdout);
    	    Lres = at51_at89c51_lock3(at51_param, &Lpro_error, &Lpro_status);
    	    if (Lres == -1) {
    	        printf("Error: device lock 3 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89C51 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89C51 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	}
	else {
    	    printf("Reading Lock bits... not exist for this device\n");
	}
    } /* end of reading lock bits AT89C51 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89C51_command() */

int at51_dec_process_device_AT89C52_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 8192;

    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89c52_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89C52 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89c52_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	}
	else {
    	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89C52 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89c52_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Erasing... not exist for this device\n");
	}
    } /* end of erasing AT89C52 */
    if (at51_param->command & _CMD_WRITE) {
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("error - see below!\n");
		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c52_write(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    	        case _S_PRO_ERRORINVALIDREQUEST:
	    		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
			case _S_PRO_ERRORWRITINGDEVICE:
			    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
			default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
	    	    }
		    return -1;
		}
	    }
	}
	else {
    	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89C52 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
    	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
    	        printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c52_burn(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
    	    printf("Burning... not exist for this device\n");
	}
    } /* end of burning AT89C52 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
    	    printf("Locking 1... ");
    	    fflush(stdout);
	    Lres = at51_at89c52_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89C52 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89c52_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89C52 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
    	    printf("Locking 3... ");
    	    fflush(stdout);
    	    Lres = at51_at89c52_lock3(at51_param, &Lpro_error, &Lpro_status);
    	    if (Lres == -1) {
    		printf("Error: device lock 3 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89C52 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89C52 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	}
	else {
    	    printf("Reading Lock bits... not exist for this device\n");
	}
    } /* end of reading lock bits AT89C52 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89C52_command() */

int at51_dec_process_device_AT89C1051_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 1024;

    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89c1051_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89C1051 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89c1051_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
	        return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
    	    if (Lres == -1) {
		printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
	        *dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	}
	else {
    	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89C1051 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89c1051_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
    	        *dec_error = Lpro_error;
	        *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
	        return -1;
	    }
	}
	else {
    	    printf("Erasing... not exist for this device\n");
	}
    } /* end of erasing AT89C1051 */
    if (at51_param->command & _CMD_WRITE) {
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("error - see below!\n");
		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
	        *dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
	        return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c1051_write(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    		case _S_PRO_ERRORINVALIDREQUEST:
	    		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
			case _S_PRO_ERRORWRITINGDEVICE:
	    		    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
			default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
		    }
		    return -1;
		}
	    }
	}
	else {
    	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89C1051 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
    	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
    		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c1051_burn(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
    	    printf("Burning... not exist for this device\n");
	}
    } /* end of burning AT89C1051 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
    	    printf("Locking 1... ");
    	    fflush(stdout);
	    Lres = at51_at89c1051_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
	        *dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89C1051 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89c1051_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89C1051 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
	}
	else {
	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89C1051 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89C1051 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	}
	else {
    	    printf("Reading Lock bits... not exist for this device\n");
	}
    } /* end of reading lock bits AT89C1051 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89C1051_command() */

int at51_dec_process_device_AT89C2051_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 2048;
    
    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89c2051_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89C2051 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89c2051_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
	        return -1;
	    }
	}
	else {
    	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89C2051 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89c2051_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
	        *dec_error = Lpro_error;
	        *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
    		return -1;
	    }
	}
	else {
    	    printf("Erasing... not exist for this device\n");
	}
    } /* end of erasing AT89C2051 */
    if (at51_param->command & _CMD_WRITE) {
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
	        printf("error - see below!\n");
	        printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
	        *dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
	        return -1;
	    }
	    if (at51_param->buffersize == 0) {
	        printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
	        Lres = at51_at89c2051_write(at51_param, &Lpro_error, &Lpro_status);
	        if (Lres == -1) {
	    	    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    	        case _S_PRO_ERRORINVALIDREQUEST:
	    	    	    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
			case _S_PRO_ERRORWRITINGDEVICE:
			    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
		        default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
		    }
		    return -1;
		}
	    }
	}
	else {
    	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89C2051 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
    	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
    	    if (Lres == -1) {
    	        printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
		*dec_error = Ldisc_error;
		*dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
		printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
		Lres = at51_at89c2051_burn(at51_param, &Lpro_error, &Lpro_status);
		if (Lres == -1) {
		    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
    	    printf("Burning... not exist for this device\n");
	}
    } /* end of burning AT89C2051 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
	    printf("Locking 1... ");
    	    fflush(stdout);
	    Lres = at51_at89c2051_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89C2051 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89c2051_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
    	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89C2051 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
	}
	else {
    	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89C2051 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89C2051 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	}
	else {
	    printf("Reading Lock bits... not exist for this device\n");
	}
    } /* end of reading lock bits AT89C2051 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89C2051_command() */

int at51_dec_process_device_AT89C4051_command(at51_param_t *at51_param,
			    int *dec_error, int *dec_status) {
    int Lres;
    int Ldisc_error, Ldisc_status;
    int Lpro_error, Lpro_status;

    printf("Note: Processing %s device\n", at51_param->partname);
    at51_param->buffersize = 4096;

    if (at51_param->command & _CMD_TEST) {
	if (at51_db[at51_param->partindex].supporttest) {
	    printf("Testing... ");
	    fflush(stdout);
	    Lres = at51_at89c4051_test(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device testing operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	}
	else {
	    printf("Testing... not exist for this device\n");
	}
    } /* end of testing AT89C4051 */
    if (at51_param->command & _CMD_READ) {
	if (at51_db[at51_param->partindex].supportread) {
	    printf("Reading... ");
	    fflush(stdout);
	    Lres = at51_at89c4051_read(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
		printf("Error: device read operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		return -1;
	    }
	    Lres = at51_disc_save_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
	        printf("Error: disc save operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
	        *dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
	        return -1;
	    }
	}
	else {
	    printf("Reading... not exist for this device\n");
	}
    } /* end of reading AT89C4051 */
    if (at51_param->command & _CMD_ERASE) {
	if (at51_db[at51_param->partindex].supporterase) {
	    printf("Erasing... ");
	    fflush(stdout);
	    Lres = at51_at89c4051_erase(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device erase operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
	        *dec_error = Lpro_error;
    	        *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
	        return -1;
	    }
	}
	else {
	    printf("Erasing... not exist for this device\n");
	}
    } /* end of erasing AT89C4051 */
    if (at51_param->command & _CMD_WRITE) {
	if (at51_db[at51_param->partindex].supportwrite) {
	    printf("Writing... ");
	    fflush(stdout);
	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
	        printf("error - see below!\n");
	        printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
	        *dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
    	        return -1;
	    }
	    if (at51_param->buffersize == 0) {
    	        printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
	        Lres = at51_at89c4051_write(at51_param, &Lpro_error, &Lpro_status);
	        if (Lres == -1) {
	    	    printf("Error: device write operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    switch(Lpro_status) {
	    		case _S_PRO_ERRORINVALIDREQUEST:
	    		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
			    break;
		        case _S_PRO_ERRORWRITINGDEVICE:
			    *dec_status = _S_DEC_ERRORWRITINGDEVICE;
			    break;
			default:
			    *dec_status = _S_DEC_ERRORWRITINGUNKNOWN;
			    break;
		    }
		    return -1;
		}
	    }
	}
	else {
	    printf("Writing... not exist for this device\n");
	}
    } /* end of writing AT89C4051 */
    if (at51_param->command & _CMD_BURN) {
	if (at51_db[at51_param->partindex].supportburn) {
    	    printf("Burning... ");
	    fflush(stdout);
    	    Lres = at51_disc_load_buffer(at51_param, &Ldisc_error, &Ldisc_status);
	    if (Lres == -1) {
		printf("Error: disc load operation: %d (%s) %d\n", Ldisc_error, strerror(Ldisc_error), Ldisc_status);
    		*dec_error = Ldisc_error;
	        *dec_status = Ldisc_status;
		return -1;
	    }
	    if (at51_param->buffersize == 0) {
	        printf(" file %s has size 0 bytes!!!\n", at51_param->filename);
	    }
	    else {
	        Lres = at51_at89c4051_burn(at51_param, &Lpro_error, &Lpro_status);
	        if (Lres == -1) {
	    	    printf("Error: device burn operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		    *dec_error = Lpro_error;
		    *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
		    return -1;
		}
	    }
	}
	else {
	    printf("Burning... not exist for this device\n");
	}
    } /* end of burning AT89C4051 */
    if (at51_param->command & _CMD_LOCK1) {
	if (at51_db[at51_param->partindex].supportlock1) {
    	    printf("Locking 1... ");
	    fflush(stdout);
    	    Lres = at51_at89c4051_lock1(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device lock 1 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
		*dec_error = Lpro_error;
		*dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
	        return -1;
	    }
	}
	else {
	    printf("Locking 1... not exist for this device\n");
	}
    } /* end of locking 1 AT89C4051 */
    if (at51_param->command & _CMD_LOCK2) {
	if (at51_db[at51_param->partindex].supportlock2) {
	    printf("Locking 2... ");
	    fflush(stdout);
	    Lres = at51_at89c4051_lock2(at51_param, &Lpro_error, &Lpro_status);
	    if (Lres == -1) {
	        printf("Error: device lock 2 operation: %d (%s) %d\n", Lpro_error, strerror(Lpro_error), Lpro_status);
	        *dec_error = Lpro_error;
	        *dec_status = _S_DEC_ERRORDEVICENOTFOUNDORDIFFERENT;
	        return -1;
    	    }
	}
	else {
	    printf("Locking 2... not exist for this device\n");
	}
    } /* end of locking 2 AT89C4051 */
    if (at51_param->command & _CMD_LOCK3) {
	if (at51_db[at51_param->partindex].supportlock3) {
	}
	else {
    	    printf("Locking 3... not exist for this device\n");
	}
    } /* end of locking 3 AT89C4051 */
    if (at51_param->command & _CMD_LOCK4) {
	if (at51_db[at51_param->partindex].supportlock4) {
	}
	else {
    	    printf("Locking 4... not exist for this device\n");
	}
    } /* end of locking 4 AT89C4051 */
    if (at51_param->command & _CMD_READLOCK) {
	if (at51_db[at51_param->partindex].supportreadlock) {
	}
	else {
    	    printf("Reading Lock bits... not exist for this device\n");
	}
    } /* end of reading lock bits AT89C4051 */
    /* prepare to exit */
    *dec_error = 0;
    *dec_status = _S_DEC_OKDEVICEPROCESSED;
    return 0;
} /* end of at51_dec_process_device_AT89C4051_command() */

/* end of at51decision.c */
