/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at51disc.c		disc save / load
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include "at51disc.h"

int at51_disc_save_buffer(at51_param_t *at51_param, 
			    int *disc_error, int *disc_status) {
    int Lfd;
    int Lres;
    
    if (strlen(at51_param->filename) == 0) {
	*disc_error = EINVAL;
	*disc_status = _S_DISC_ERRORINVALIDREQUEST;
	return -1;
    }
    Lfd = open(at51_param->filename, O_CREAT | O_RDWR, 
	S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
    if (Lfd == -1) {
	*disc_error = errno;
	*disc_status = _S_DISC_ERRORCREATEFILE;
	return -1;
    }
    Lres = write(Lfd, at51_param->buffer, at51_param->buffersize);
    if (Lres == -1) {
	*disc_error = errno;
	*disc_status = _S_DISC_ERRORWRITEFILE;
	close (Lfd);
	return -1;
    }
    if (Lres != at51_param->buffersize) {
	*disc_error = errno;
	*disc_status = _S_DISC_ERRORWRITEFILE;
	close (Lfd);
	return -1;
    }
    close (Lfd);
    *disc_error = 0;
    *disc_status = _S_DISC_OKWRITEFILE;
    return 0;
} /* end of at51_disc_save_buffer() */

int at51_disc_load_buffer(at51_param_t *at51_param,
			    int *disc_error, int *disc_status) {
    int Lfd;
    int Lres;
    
    if (strlen(at51_param->filename) == 0) {
	*disc_error = EINVAL;
	*disc_status = _S_DISC_ERRORINVALIDREQUEST;
	return -1;
    }
    Lfd = open(at51_param->filename, O_RDONLY);
    if (Lfd == -1) {
	*disc_error = errno;
	*disc_status = _S_DISC_ERROROPENFILE;
	return -1;
    }
    Lres = read(Lfd, at51_param->buffer, at51_param->buffersize);
    if (Lres == -1) {
	*disc_error = errno;
	*disc_status = _S_DISC_ERRORREADFILE;
	close (Lfd);
	return -1;
    }
    if (Lres != at51_param->buffersize) 
	at51_param->buffersize = Lres;
    close (Lfd);
    *disc_error = 0;
    *disc_status = _S_DISC_OKWRITEFILE;
    return 0;
} /* end of at51_disc_load_buffer() */

/* end of at51disc.c */
