/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at51help.c		help routines
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include "at51help.h"

int at51_help_show_help_full(at51_param_t *at51_param) {
    int Lpe, Lps;
    if (at51_param->showhelp == _Yes) {
	at51_help_show_help_basic(at51_param);
	at51_help_show_help_part(at51_param);
	at51_help_show_help_command(at51_param);
	at51_help_show_help_part_and_command(at51_param);
	if (at51_param_finalize_struct(at51_param, &Lpe, &Lps) == -1) {
	    printf("Error: " _AT51_PROGRAM_NAME " finalization struct error: %d (%s) %d\n",
		Lpe, strerror(Lpe), Lps);
	}
	exit(0);
    }
    if (at51_param->showlist == _Yes) {
	at51_help_show_help_intro(at51_param);    
	printf("\n");
	at51_help_show_list_supported_commands(at51_param);
	printf("\n");
	at51_help_show_list_supported_devices(at51_param);
	if (at51_param_finalize_struct(at51_param, &Lpe, &Lps) == -1) {
	    printf("Error: " _AT51_PROGRAM_NAME " finalization struct error: %d (%s) %d\n",
		Lpe, strerror(Lpe), Lps);
	}
	exit(0);
    }
    return 0;
} /* end of at51_help_show_help_full() */

int at51_help_show_help_intro(at51_param_t *at51_param) {
    printf (_AT51_PROGRAM_NAME" is a basic part from package "_AT51TOOL_PACKAGE_NAME" version "_AT51TOOL_PACKAGE_VERSION"\n");
    printf ("Summary page: http://www.sourceforge.net/projects/at51programmer\n");
    printf ("Home page: http://at51programmer.sourceforge.net\n");
    return 0;
} /* end of at51_help_show_help_intro() */

int at51_help_show_help_basic(at51_param_t *at51_param) {
    if ((strlen(at51_param->partname) == 0) && 
	(at51_param->command == 0)) {
	at51_help_show_help_intro(at51_param);
	printf ("Basic help!\n\n");
	printf ("Usage:\n");
	printf ("         "_AT51_PROGRAM_NAME" -p <part_name>  -f <file_name>\n");
	printf ("              < -t | -r | -w | -e | -b | -1 | -2 | -3 | -4 | -0 >\n");
	printf ("              [ -F ]  [ -v ]  [ -l ]  [ -h ]\n");
    }
    return 0;
} /* end of at51_help_show_help_basic() */

int at51_help_show_help_part(at51_param_t *at51_param) {
    if ((strlen(at51_param->partname) > 0) && 
	(at51_param->command == 0)) {
	at51_help_show_help_intro(at51_param);
	if (at51_param->partindex == -1) {
	    printf("Error: Device is not supported, update your "_AT51TOOL_PACKAGE_NAME" !\n");
	    return 0;
	}
	printf("Note: Supported commands for this device: \n");
	if (at51_db[at51_param->partindex].supporttest == _Yes)
	    printf ("test command - %s: %s\n",
		at51_param->partname, at51_db[at51_param->partindex].testhelp);
	if (at51_db[at51_param->partindex].supportread == _Yes)
	    printf ("read command - %s: %s\n",
		at51_param->partname, at51_db[at51_param->partindex].readhelp);
	if (at51_db[at51_param->partindex].supportwrite == _Yes)
	    printf ("write command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].writehelp);
	if (at51_db[at51_param->partindex].supportburn == _Yes)
	    printf ("burn command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].burnhelp);
	if (at51_db[at51_param->partindex].supporterase == _Yes)
	    printf ("erase command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].erasehelp);
	if (at51_db[at51_param->partindex].supportlock1 == _Yes)
	    printf ("lock 1 command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].lock1help);
	if (at51_db[at51_param->partindex].supportlock2 == _Yes)
	    printf ("lock 2 command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].lock2help);
	if (at51_db[at51_param->partindex].supportlock3 == _Yes)
	    printf ("lock 3 command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].lock3help);
	if (at51_db[at51_param->partindex].supportlock4 == _Yes)
	    printf ("lock 4 command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].lock4help);
	if (at51_db[at51_param->partindex].supportreadlock == _Yes)
	    printf ("read lock command - %s: %s\n",
	        at51_param->partname, at51_db[at51_param->partindex].readlockhelp);
    }
    return 0;
} /* end of at51_help_show_help_part() */

int at51_help_show_help_command(at51_param_t *at51_param) {
    if ((strlen(at51_param->partname) == 0) && 
	(at51_param->command != 0)) {
	at51_help_show_help_intro(at51_param);
	if (at51_param->command & _CMD_TEST)
	    printf ("Note: Command test - test device, get and check device id\n");
	if (at51_param->command & _CMD_READ)
	    printf ("Note: Command read - read device memory and store it in file\n");
	if (at51_param->command & _CMD_WRITE)
	    printf ("Note: Command write - write and verify device memory from file \n");
	if (at51_param->command & _CMD_BURN)
	    printf ("Note: Command burn - write and not verify (not recommended) \n");
	if (at51_param->command & _CMD_ERASE)
	    printf ("Note: Command erase - erase device memory\n");
	if (at51_param->command & _CMD_LOCK1)
	    printf ("Note: Command lock 1 - lock device memory level 1 \n");
	if (at51_param->command & _CMD_LOCK2)
	    printf ("Note: Command lock 2 - lock device memory level 2 \n");
	if (at51_param->command & _CMD_LOCK3)
	    printf ("Note: Command lock 3 - lock device memory level 3 \n");
	if (at51_param->command & _CMD_LOCK4)
	    printf ("Note: Command lock 4 - lock device memory level 4 \n");
	if (at51_param->command & _CMD_READLOCK)
	    printf ("Note: Command read lock - read device lock bits \n");
    }
    return 0;
} /* end of at51_help_show_help_command() */

int at51_help_show_help_part_and_command(at51_param_t *at51_param) {
    if ((strlen(at51_param->partname) > 0) && 
	(at51_param->command != 0)) {
	at51_help_show_help_intro(at51_param);
	if (at51_param->partindex == -1) {
	    printf("Error: Device is not supported, update your "_AT51TOOL_PACKAGE_NAME" !\n");
	    return 0;
	}
	if (at51_param->command & _CMD_TEST) {
	    if (at51_db[at51_param->partindex].supporttest == _Yes)
		printf ("test command supported! %s: %s\n", 
		    at51_param->partname, at51_db[at51_param->partindex].testhelp);
	    else
		printf ("test command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].testhelp);
	}
	if (at51_param->command & _CMD_READ) {
	    if (at51_db[at51_param->partindex].supportread == _Yes)
		printf ("read command supported! %s: %s\n", 
		    at51_param->partname, at51_db[at51_param->partindex].readhelp);
	    else
		printf ("read command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].readhelp);
	}
	if (at51_param->command & _CMD_WRITE) {
	    if (at51_db[at51_param->partindex].supportwrite == _Yes)
		printf ("write command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].writehelp);
	    else
		printf ("write command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].writehelp);
	}
	if (at51_param->command & _CMD_BURN) {
	    if (at51_db[at51_param->partindex].supportburn == _Yes)
		printf ("burn command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].burnhelp);
	    else
		printf ("burn command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].burnhelp);
	}
	if (at51_param->command & _CMD_ERASE) {
	    if (at51_db[at51_param->partindex].supporterase == _Yes)
		printf ("erase command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].erasehelp);
	    else
		printf ("erase command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].erasehelp);
	}
	if (at51_param->command & _CMD_LOCK1) {
	    if (at51_db[at51_param->partindex].supportlock1 == _Yes)
		printf ("lock 1 command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock1help);
	    else
		printf ("lock 1 command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock1help);
	}
	if (at51_param->command & _CMD_LOCK2) {
	    if (at51_db[at51_param->partindex].supportlock2 == _Yes)
		printf ("lock 2 command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock2help);
	    else
		printf ("lock 2 command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock2help);
	}
	if (at51_param->command & _CMD_LOCK3) {
	    if (at51_db[at51_param->partindex].supportlock3 == _Yes)
		printf ("lock 3 command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock3help);
	    else
		printf ("lock 3 command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock3help);
	}
	if (at51_param->command & _CMD_LOCK4) {
	    if (at51_db[at51_param->partindex].supportlock4 == _Yes)
		printf ("lock 4 command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock4help);
	    else
		printf ("lock 4 command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].lock4help);
	}
	if (at51_param->command & _CMD_READLOCK) {
	    if (at51_db[at51_param->partindex].supportreadlock == _Yes)
		printf ("read lock command supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].readlockhelp);
	    else
		printf ("read lock command not supported! %s: %s\n",
		    at51_param->partname, at51_db[at51_param->partindex].readlockhelp);
	}
    }
    return 0;
} /* end of at51_help_show_help_part_and_command() */

int at51_help_show_help_compillance(at51_param_t *at51_param) {
    int Lpe, Lps;

    at51_help_show_help_intro(at51_param);
    printf("Error: Not compillant commands!\n");
    printf("Do not use commands:");
    if (at51_param->command & _CMD_TEST) printf (" TEST ");
    if (at51_param->command & _CMD_READ) printf (" READ ");
    if (at51_param->command & _CMD_WRITE) printf (" WRITE ");
    if (at51_param->command & _CMD_BURN) printf (" BURN ");
    if (at51_param->command & _CMD_ERASE) printf (" ERASE ");
    if (at51_param->command & _CMD_LOCK1) printf (" LOCK1 ");
    if (at51_param->command & _CMD_LOCK2) printf (" LOCK2 ");
    if (at51_param->command & _CMD_LOCK3) printf (" LOCK3 ");
    if (at51_param->command & _CMD_LOCK4) printf (" LOCK4 ");
    if (at51_param->command & _CMD_READLOCK) printf (" READLOCK ");
    printf("together!\n");
    if (at51_param_finalize_struct(at51_param, &Lpe, &Lps) == -1) {
	printf("Error: " _AT51_PROGRAM_NAME " finalization struct error: %d (%s) %d\n",
	    Lpe, strerror(Lpe), Lps);
    }
    exit(0);
} /* end of at51_help_show_help_compillance() */

int at51_help_show_help_device_isnt_supported(at51_param_t *at51_param) {
    if (strlen(at51_param->partname) && (at51_param->partindex == -1)) {
	at51_help_show_help_intro(at51_param);
	printf("Error: Device is not supported, update your "_AT51TOOL_PACKAGE_NAME" !\n");
	exit(0);
    }
    return 0;
} /* end of at51_help_show_help_device_isnt_supported() */

int at51_help_show_list_supported_devices(at51_param_t *at51_param) {
    int Li = 0;

    printf(" List of all supported devices\n");
    printf("-----------------------------------------------------------------------\n");
    printf("                       part name                          vendor\n");
    printf("-----------------------------------------------------------------------\n");
    while (strlen(at51_db[Li].partname)) {
	printf("%32s%32s\n", at51_db[Li].partname, at51_db[Li].vendor);
	Li++;
    }
    printf("-----------------------------------------------------------------------\n");
    return 0;
} /* end of at51_help_show_list_supported_devices() */

int at51_help_show_list_supported_commands(at51_param_t *at51_param) {

    printf(" List of all supported commands\n");
    printf("-----------------------------------------------------------------------\n");
    printf(" command       parameter       description                             \n");
    printf("-----------------------------------------------------------------------\n");
    printf(" test          -t              test device, get and check device id    \n");
    printf(" read          -r              read device memory and store it in file \n");
    printf(" write         -w              write and verify device memory from file\n");
    printf(" burn          -b              write and not verify device memory      \n");
    printf(" erase         -e              erase device memory                     \n");
    printf(" lock 1        -1              lock device memory level 1              \n");
    printf(" lock 2        -2              lock device memory level 2              \n");
    printf(" lock 3        -3              lock device memory level 3              \n");
    printf(" lock 4        -4              lock device memory level 4              \n");
    printf(" read lock     -0              read device lock bits                   \n");
    printf("-----------------------------------------------------------------------\n");
    return 0;    
} /* end of at51_help_show_list_supported_commands() */

/* end of at51help.c */
