/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at51param.c		at51param struct processing functions
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include "at51param.h"

int at51_param_init_struct(at51_param_t *at51_param,
			    int *param_error, int *param_status) {
    int Li;

    if (at51_param == NULL) {
	*param_error = EINVAL;
	*param_status = _S_PARAM_ERRORINVALIDSTRUCT;
	return -1;
    }
    at51_param->partname[0] = 0;
    at51_param->command = 0;
    at51_param->forcedcommand = _No;
    at51_param->showhelp = 0;
    at51_param->showlist = 0;
    at51_param->partindex = -1;
    strcpy(at51_param->filename,"./device.bin");
    at51_param->buffersize = 0;
    /* Allocate memory for buffer */
    at51_param->buffer = (char *)malloc(2048*1024*sizeof(char));
    if (at51_param->buffer == NULL) {
	*param_error = errno;
	*param_status = _S_PARAM_ERRORALLOCATEMEMORY;
	return -1;
    }
    /* clear buffer */
    for (Li = 0; Li < 2048*1024; Li++) at51_param->buffer[Li] = 0xFF;
    /* Open serial device */
    at51_param->serial_fd = open("/dev/ttyS0", O_RDWR | O_NOCTTY);
    if (at51_param->serial_fd == -1) {
	*param_error = errno;
	*param_status = _S_PARAM_ERROROPENSERIALPORT;
	return -1;
    }
    tcgetattr(at51_param->serial_fd, &at51_param->oldtio);
    bzero(&at51_param->newtio, sizeof(struct termios));
    at51_param->newtio.c_cflag = B57600 | CS8 | CLOCAL | CREAD;
    at51_param->newtio.c_iflag = IGNPAR | IXON | IXOFF;
    at51_param->newtio.c_oflag = 0;
    at51_param->newtio.c_lflag = 0;
    at51_param->newtio.c_cc[VTIME] = 0;
    at51_param->newtio.c_cc[VMIN] = 1;
    tcflush(at51_param->serial_fd, TCOFLUSH);
    tcsetattr(at51_param->serial_fd, TCSANOW, &at51_param->newtio);
    /* prepare to exit */
    *param_error = 0;
    *param_status = _S_PARAM_OKINITIALIZATION;
    return 0;
} /* end of at51_param_init_struct() */

int at51_param_finalize_struct(at51_param_t *at51_param,
			    int *param_error, int *param_status) {
    if (at51_param == NULL) {
	*param_error = EINVAL;
	*param_status = _S_PARAM_ERRORINVALIDSTRUCT;
	return -1;
    }
    /* free memory */
    free(at51_param->buffer);
    /* restore serial port previous state */
    tcflush(at51_param->serial_fd, TCOFLUSH);
    tcsetattr(at51_param->serial_fd, TCSANOW, &at51_param->oldtio);
    /* close serial port */
    close(at51_param->serial_fd);
    /* prepare to exit */
    *param_error = 0;
    *param_status = _S_PARAM_OKFINALIZATION;
    return 0;
} /* end of at51_param_finalize_struct() */

/* end of at51param.c */
