/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at89c1051.c		AT89C1051 programming functions
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include "at89c1051.h"

int at51_at89c1051_test(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	_at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	*pro_status = _S_PRO_ERRORINVALIDREQUEST;
	*pro_error = EINVAL;
	return -1;
    }
    printf("(%2.2X %2.2X) \n", LID1, LID2);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    return 0;
} /* end of at51_at89c1051_test() */

int at51_at89c1051_read(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    int Li, Lj;
    unsigned char LID1, LID2;
    int Lstep;
    int Lval = 0;
    int Lsectorsnum;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	if (at51_param->forcedcommand == _No) {
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced reading ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X) ", LID1, LID2);
    /* Main reading routines */
    /* Reset device! */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* prepare to read */
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Set_P35(at51_param->serial_fd);
    _at51_board2051_Set_P37(at51_param->serial_fd);
    if (at51_param->buffersize % 256 == 0)
	Lsectorsnum = at51_param->buffersize / 256;
    else
	Lsectorsnum = (at51_param->buffersize / 256) + 1;
    Lstep = 100 / Lsectorsnum;
    Lval = 0;
    for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	Lval += Lstep;
	printf("%2.2d %c", Lval, 0x25);
	fflush(stdout);
	for (Li = 0; Li < 256; Li++) {
	    at51_param->buffer[(Lj * 256) + Li] = _at51_board2051_ReceiveData(at51_param->serial_fd);
	    _at51_board2051_Set_XTAL(at51_param->serial_fd);
	    _at51_board2051_Clear_XTAL(at51_param->serial_fd);
	}
	printf("%c%c%c%c", 8, 8, 8, 8);
	fflush(stdout);
    }
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    /* Power Off */
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKREAD;
    return 0;
} /* end of at51_at89c1051_read() */

int at51_at89c1051_erase(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	if (at51_param->forcedcommand == _No) {
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced erasing ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X) ", LID1, LID2);
    /* Main erasing routines */
    /* Reset device! */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* prepare to erase */
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Set_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    /* erasing pulses */
    _at51_board2051_Prog12_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Set_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKERASE;
    return 0;
} /* end of at51_at89c1051_erase() */

int at51_at89c1051_write(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    int Li, Lj;
    unsigned char LID1, LID2;
    int Lstep;
    int Lval = 0;
    int Lsectorsnum;
    int Lerrors = 0;
    int Ldifferent = _No;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	if (at51_param->forcedcommand == _No) {
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced writing ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X) ", LID1, LID2);
    /* Main writing routines */
    while(1) {
	/* prepare for successful writing */
	Ldifferent = _No;
	/* Reset device! */
	at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
	at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
	_at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
	/* prepare to burn */
	_at51_board2051_Set_PROG(at51_param->serial_fd);
	_at51_board2051_Clear_P33(at51_param->serial_fd);
	_at51_board2051_Set_P34(at51_param->serial_fd);
	_at51_board2051_Set_P35(at51_param->serial_fd);
	_at51_board2051_Set_P37(at51_param->serial_fd);
	if (at51_param->buffersize % 256 == 0)
	    Lsectorsnum = at51_param->buffersize / 256;
	else
	    Lsectorsnum = (at51_param->buffersize / 256) + 1;
	Lstep = 100 / Lsectorsnum;
	Lval = 0;
	for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	    Lval += Lstep;
	    printf("%2.2d %c", Lval, 0x25);
	    fflush(stdout);
	    for (Li = 0; Li < 256; Li++) {
		_at51_board2051_SendData(at51_param->serial_fd, at51_param->buffer[(Lj * 256) + Li]);
		_at51_board2051_Prog12_On(at51_param->serial_fd);
		_at51_board2051_Clear_PROG(at51_param->serial_fd);
		_at51_board2051_Set_PROG(at51_param->serial_fd);
		_at51_board2051_Prog12_Off(at51_param->serial_fd);
		_at51_board2051_Set_XTAL(at51_param->serial_fd);
		_at51_board2051_Clear_XTAL(at51_param->serial_fd);
	    }
	    printf("%c%c%c%c", 8, 8, 8, 8);
	    fflush(stdout);
	}
	_at51_board2051_Clear_P34(at51_param->serial_fd);
	_at51_board2051_Clear_P35(at51_param->serial_fd);
	_at51_board2051_Clear_P37(at51_param->serial_fd);
	/* continue with verifying */
	printf("%c100 %c verifying... ", 8, 0x25);
	/* Reset device! */
	at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
	at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd);
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); sleep(2);
	_at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
	/* prepare to read */
	_at51_board2051_Set_PROG(at51_param->serial_fd);
	_at51_board2051_Clear_P33(at51_param->serial_fd);
	_at51_board2051_Clear_P34(at51_param->serial_fd);
	_at51_board2051_Set_P35(at51_param->serial_fd);
	_at51_board2051_Set_P37(at51_param->serial_fd);
	if (at51_param->buffersize % 256 == 0)
	    Lsectorsnum = at51_param->buffersize / 256;
	else
	    Lsectorsnum = (at51_param->buffersize / 256) + 1;
	Lstep = 100 / Lsectorsnum;
	Lval = 0;
	for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	    Lval += Lstep;
	    printf("%2.2d %c", Lval, 0x25);
	    fflush(stdout);
	    for (Li = 0; Li < 256; Li++) {
		if (at51_param->buffer[(Lj * 256) + Li] != _at51_board2051_ReceiveData(at51_param->serial_fd))
		    Ldifferent = _Yes;
		_at51_board2051_Set_XTAL(at51_param->serial_fd);
		_at51_board2051_Clear_XTAL(at51_param->serial_fd);
		if (Ldifferent == _Yes) break;
	    }
	    printf("%c%c%c%c", 8, 8, 8, 8);
	    fflush(stdout);
	    if (Ldifferent == _Yes) break;
	}
	_at51_board2051_Clear_P35(at51_param->serial_fd);
	_at51_board2051_Clear_P37(at51_param->serial_fd);
	printf("%c100 %c", 8, 0x25);
	if (Ldifferent == _No) break;
	Lerrors++;
	if (Lerrors == 5) {
	    /* Power Off */
	    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    /* Prepare to exit */
	    printf(" done!\n");
	    *pro_error = EINVAL;
	    *pro_status = _S_PRO_ERRORWRITINGDEVICE;
	    return -1;
	}
	printf("\nTrying to rewrite (%d/4)...", Lerrors);
	/* sleep 1 sec */
	sleep(2);
    } /* end of while (1) write and verify cycle */
    /* Power Off */
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKWRITE;
    return 0;
} /* end of at51_at89c1051_write() */

int at51_at89c1051_burn(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    int Li, Lj;
    unsigned char LID1, LID2;
    int Lstep;
    int Lval = 0;
    int Lsectorsnum;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	if (at51_param->forcedcommand == _No) {
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced burning ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X) ", LID1, LID2);
    /* Main burning routines */
    /* Reset device! */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* prepare to burn */
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Set_P34(at51_param->serial_fd);
    _at51_board2051_Set_P35(at51_param->serial_fd);
    _at51_board2051_Set_P37(at51_param->serial_fd);
    if (at51_param->buffersize % 256 == 0)
	Lsectorsnum = at51_param->buffersize / 256;
    else
	Lsectorsnum = (at51_param->buffersize / 256) + 1;
    Lstep = 100 / Lsectorsnum;
    Lval = 0;
    for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	Lval += Lstep;
	printf("%2.2d %c", Lval, 0x25);
	fflush(stdout);
	for (Li = 0; Li < 256; Li++) {
	    _at51_board2051_SendData(at51_param->serial_fd, at51_param->buffer[(Lj * 256) + Li]);
	    _at51_board2051_Prog12_On(at51_param->serial_fd);
	    _at51_board2051_Clear_PROG(at51_param->serial_fd);
	    _at51_board2051_Set_PROG(at51_param->serial_fd);
	    _at51_board2051_Prog12_Off(at51_param->serial_fd);
	    _at51_board2051_Set_XTAL(at51_param->serial_fd);
	    _at51_board2051_Clear_XTAL(at51_param->serial_fd);
	}
	printf("%c%c%c%c", 8, 8, 8, 8);
	fflush(stdout);
    }
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    /* Power Off */
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKWRITE;
    return 0;
} /* end of at51_at89c1051_burn() */

int at51_at89c1051_lock1(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	if (at51_param->forcedcommand == _No) {
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced locking 1 ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X) ", LID1, LID2);
    /* Main lock 1 routines */
    /* Reset device! */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* prepare to lock level 1 */
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Set_P33(at51_param->serial_fd);
    _at51_board2051_Set_P34(at51_param->serial_fd);
    _at51_board2051_Set_P35(at51_param->serial_fd);
    _at51_board2051_Set_P37(at51_param->serial_fd);
    /* locking 1 pulses */
    _at51_board2051_Prog12_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(5000);
    _at51_board2051_Set_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKLOCK1;
    return 0;
} /* end of at51_at89c1051_lock1() */

int at51_at89c1051_lock2(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2;
    
    /* Power On and Reset device */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Sleep 1 sec */
    sleep(1);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board2051_SendData(at51_param->serial_fd, 0xFF);
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    LID1 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    _at51_board2051_Set_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    LID2 = _at51_board2051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x11)) {
	/* Device not found or different */
	_at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X (1E 11)\n",
	    LID1, LID2);
	if (at51_param->forcedcommand == _No) {
	    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced locking 2 ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X) ", LID1, LID2);
    /* Main lock 2 routines */
    /* Reset device! */
    at51_board2051_Power_Off(at51_param->serial_fd); usleep(100000);
    at51_board2051_Prog_Off(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(100000);
    _at51_board2051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* prepare to lock level 2 */
    _at51_board2051_Set_PROG(at51_param->serial_fd);
    _at51_board2051_Set_P33(at51_param->serial_fd);
    _at51_board2051_Set_P34(at51_param->serial_fd);
    _at51_board2051_Clear_P35(at51_param->serial_fd);
    _at51_board2051_Clear_P37(at51_param->serial_fd);
    /* locking 2 pulses */
    _at51_board2051_Prog12_On(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(5000);
    _at51_board2051_Set_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board2051_Clear_P33(at51_param->serial_fd);
    _at51_board2051_Clear_P34(at51_param->serial_fd);
    _at51_board2051_Clear_XTAL(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Clear_PROG(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board2051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKLOCK2;
    return 0;
} /* end of at51_at89c1051_lock2() */

/* end of at89c1051.c */
