/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * at89s51.c		AT89S51 programming functions
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include "at89s51.h"

int at51_at89s51_test(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2, LID3;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	_at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	*pro_status = _S_PRO_ERRORINVALIDREQUEST;
	*pro_error = EINVAL;
	return -1;
    }
    printf("(%2.2X %2.2X %2.2X) \n", LID1, LID2, LID3);
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    return 0;
} /* end of int at51_at89s51_test() */

int at51_at89s51_read(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    int Li, Lj;
    unsigned char LID1, LID2, LID3;
    int Lstep;
    int Lval = 0;
    int Lsectorsnum;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced reading ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main reading routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P37(at51_param->serial_fd); usleep(1000);
    if (at51_param->buffersize % 256 == 0)
	Lsectorsnum = at51_param->buffersize / 256;
    else
	Lsectorsnum = (at51_param->buffersize / 256) + 1;
    Lstep = 100 / Lsectorsnum;
    Lval = 0;
    for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	Lval += Lstep;
	if (Lval >= 100) Lval = 99;
	printf("%2.2d %c", Lval, 0x25);
	fflush(stdout);
	if (Lj & 0x01) set_m_p22(at51_param->serial_fd); /* A8 */
	else clear_m_p22(at51_param->serial_fd);
	if (Lj & 0x02) set_m_p23(at51_param->serial_fd); /* A9 */
	else clear_m_p23(at51_param->serial_fd);
	if (Lj & 0x04) set_m_p20(at51_param->serial_fd); /* A10 */
	else clear_m_p20(at51_param->serial_fd);
	if (Lj & 0x08) set_m_p21(at51_param->serial_fd); /* A11 */
	else clear_m_p21(at51_param->serial_fd);
	for (Li = 0; Li < 256; Li++) {
	    _at51_board8051_SendAddress8(at51_param->serial_fd,Li); /* A0 - A7 */
    	    at51_param->buffer[(Lj * 256) + Li] = _at51_board8051_ReceiveData(at51_param->serial_fd);
	}
	printf("%c%c%c%c", 8, 8, 8, 8);
	fflush(stdout);
    }
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKREAD;
    return 0;
} /* end of at51_at89s51_read() */

int at51_at89s51_erase(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2, LID3;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced erasing ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main erasing routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF); usleep(1000);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_On(at51_param->serial_fd); sleep(1);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_ALE(at51_param->serial_fd); usleep(100);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKERASE;
    return 0;
} /* end of at51_at89s51_erase() */

int at51_at89s51_write(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    int Li, Lj;
    unsigned char LID1, LID2, LID3;
    int Lstep;
    int Lval = 0;
    int Lsectorsnum;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced writing ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main writing routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P37(at51_param->serial_fd); usleep(1000);
    if (at51_param->buffersize % 256 == 0)
	Lsectorsnum = at51_param->buffersize / 256;
    else
	Lsectorsnum = (at51_param->buffersize / 256) + 1;
    Lstep = 100 / Lsectorsnum;
    Lval = 0;
    for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	Lval += Lstep;
	if (Lval >= 100) Lval = 99;
	printf("%2.2d %c", Lval, 0x25);
	fflush(stdout);
	if (Lj & 0x01) set_m_p22(at51_param->serial_fd); /* A8 */
	else clear_m_p22(at51_param->serial_fd);
	if (Lj & 0x02) set_m_p23(at51_param->serial_fd); /* A9 */
	else clear_m_p23(at51_param->serial_fd);
	if (Lj & 0x04) set_m_p20(at51_param->serial_fd); /* A10 */
	else clear_m_p20(at51_param->serial_fd);
	if (Lj & 0x08) set_m_p21(at51_param->serial_fd); /* A11 */
	else clear_m_p21(at51_param->serial_fd);
	for (Li = 0; Li < 256; Li++) {
	    _at51_board8051_SendAddress8(at51_param->serial_fd, Li); /* A0 - A7 */
    	    _at51_board8051_SendData(at51_param->serial_fd, at51_param->buffer[(Lj * 256) + Li]);
	    _at51_board8051_Set_P27(at51_param->serial_fd);
	    _at51_board8051_Set_P33(at51_param->serial_fd);
	    _at51_board8051_Prog12_On(at51_param->serial_fd);
	    _at51_board8051_Set_ALE(at51_param->serial_fd);
	    _at51_board8051_Clear_ALE(at51_param->serial_fd);
	    _at51_board8051_Set_ALE(at51_param->serial_fd);
	    _at51_board8051_Prog12_Off(at51_param->serial_fd);
    	    _at51_board8051_SendData(at51_param->serial_fd, 0xFF);
	    _at51_board8051_Clear_P27(at51_param->serial_fd);
	    _at51_board8051_Clear_P33(at51_param->serial_fd);
	    if (at51_param->buffer[(Lj * 256) + Li] != _at51_board8051_ReceiveData(at51_param->serial_fd)) {
		_at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
		_at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
		/* Power Off */
		_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
		_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
		_at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
		*pro_error = 0;
		*pro_status = _S_PRO_ERRORWRITINGDEVICE;
    		return -1;
	    }
	}
	printf("%c%c%c%c", 8, 8, 8, 8);
	fflush(stdout);
    }
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKWRITE;
    return 0;
} /* end of at51_at89s51_write() */

int at51_at89s51_burn(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    int Li, Lj;
    unsigned char LID1, LID2, LID3;
    int Lstep;
    int Lval = 0;
    int Lsectorsnum;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced burning ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main burning routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P37(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P27(at51_param->serial_fd);
    _at51_board8051_Set_P33(at51_param->serial_fd);
    if (at51_param->buffersize % 256 == 0)
	Lsectorsnum = at51_param->buffersize / 256;
    else
	Lsectorsnum = (at51_param->buffersize / 256) + 1;
    Lstep = 100 / Lsectorsnum;
    Lval = 0;
    for (Lj = 0; Lj < Lsectorsnum; Lj++) {
	Lval += Lstep;
	if (Lval >= 100) Lval = 99;
	printf("%2.2d %c", Lval, 0x25);
	fflush(stdout);
	if (Lj & 0x01) set_m_p22(at51_param->serial_fd); /* A8 */
	else clear_m_p22(at51_param->serial_fd);
	if (Lj & 0x02) set_m_p23(at51_param->serial_fd); /* A9 */
	else clear_m_p23(at51_param->serial_fd);
	if (Lj & 0x04) set_m_p20(at51_param->serial_fd); /* A10 */
	else clear_m_p20(at51_param->serial_fd);
	if (Lj & 0x08) set_m_p21(at51_param->serial_fd); /* A11 */
	else clear_m_p21(at51_param->serial_fd);
	for (Li = 0; Li < 256; Li++) {
	    _at51_board8051_SendAddress8(at51_param->serial_fd, Li); /* A0 - A7 */
    	    _at51_board8051_SendData(at51_param->serial_fd, at51_param->buffer[(Lj * 256) + Li]);
	    _at51_board8051_Prog12_On(at51_param->serial_fd);
	    _at51_board8051_Set_ALE(at51_param->serial_fd);
	    _at51_board8051_Clear_ALE(at51_param->serial_fd);
	    _at51_board8051_Set_ALE(at51_param->serial_fd);
	    _at51_board8051_Prog12_Off(at51_param->serial_fd);
	}
	printf("%c%c%c%c", 8, 8, 8, 8);
	fflush(stdout);
    }
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKWRITE;
    return 0;
} /* end of at51_at89s51_burn() */

int at51_at89s51_lock1(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2, LID3;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced locking 1 ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main lock 1 routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF); usleep(1000);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P37(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_On(at51_param->serial_fd); sleep(1);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_ALE(at51_param->serial_fd); usleep(100);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKLOCK1;
    return 0;
} /* end of at51_at89s51_lock1() */

int at51_at89s51_lock2(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2, LID3;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced locking 2 ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main lock 2 routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF); usleep(1000);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_On(at51_param->serial_fd); sleep(1);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_ALE(at51_param->serial_fd); usleep(100);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKLOCK2;
    return 0;
} /* end of at51_at89s51_lock2() */

int at51_at89s51_lock3(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2, LID3;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced locking 3 ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main lock 3 routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF); usleep(1000);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_On(at51_param->serial_fd); sleep(1);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_ALE(at51_param->serial_fd); usleep(100);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog12_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    printf(" done!\n");
    *pro_error = 0;
    *pro_status = _S_PRO_OKLOCK3;
    return 0;
} /* end of at51_at89s51_lock3() */

int at51_at89s51_readlock(at51_param_t *at51_param,
			    int *pro_error, int *pro_status) {
    unsigned char LID1, LID2, LID3;
    unsigned char Llock;
    
    /* Power On */
    _at51_board8051_ActivityLED_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_On(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Prog5_On(at51_param->serial_fd); usleep(1000);
    /* Test if the device exist and if it is correct */
    /* Get device IDs */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    at51_board8051_Address(at51_param->serial_fd, 0x0000);
    LID1 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0100);
    LID2 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    at51_board8051_Address(at51_param->serial_fd, 0x0200);
    LID3 = _at51_board8051_ReceiveData(at51_param->serial_fd);
    if ((LID1 != 0x1E) || (LID2 != 0x51) || (LID3 != 0x06)) {
	/* Device not found or different */
	/* Power Off */
	_at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
	_at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
	printf("error! Device not found or not correct - %2.2X %2.2X %2.2X (1E 51 06)\n",
	    LID1, LID2, LID3);
	if (at51_param->forcedcommand == _No) {
	    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
	    *pro_status = _S_PRO_ERRORINVALIDREQUEST;
	    *pro_error = EINVAL;
	    return -1;
	}
	else
	    printf("forced reading lock ");
    }
    *pro_error = 0;
    *pro_status = _S_PRO_OKGETID;
    printf("(%2.2X %2.2X %2.2X) ", LID1, LID2, LID3);
    /* Main read lock routines */
    _at51_board8051_SendData(at51_param->serial_fd,0xFF); usleep(1000);
    _at51_board8051_Set_RST(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_PSEN(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_ALE(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P33(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Set_P36(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P37(at51_param->serial_fd); usleep(1000);
    Llock = _at51_board8051_ReceiveData(at51_param->serial_fd);
    _at51_board8051_Clear_P26(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P27(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Clear_P36(at51_param->serial_fd); usleep(1000);
    /* Power Off */
    _at51_board8051_Prog5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_Power5_Off(at51_param->serial_fd); usleep(1000);
    _at51_board8051_ActivityLED_Off(at51_param->serial_fd); usleep(1000);
    /* Prepare to exit */
    if (Llock & 0x04) printf("lock 1 - YES; ");
    else printf("lock 1 - NO; ");
    if (Llock & 0x08) printf("lock 2 - YES; ");
    else printf("lock 2 - NO; ");
    if (Llock & 0x10) printf("lock 3 - YES...");
    else printf("lock 3 - NO...");
    printf(" done!\n");
    *pro_error = 0;
    if ((Llock & 0x04) || (Llock & 0x08) || (Llock & 0x10)) {
	printf("Note: The device memory is locked...\n");
	*pro_status = _S_PRO_OKDEVICELOCKED;
    }
    else 
	*pro_status = _S_PRO_OKREADLOCK;
    return 0;
} /* end of at51_at89s51_readlock() */

/* end of at89s51.c */
