/*
 * at51programmer       Programmer for Atmel 8051 devices, Parallel Flash and
 *                      E2Prom memories.
 *
 * disable_communication.c
 *
 * Copyright (C) 2004, Petar T. Petkoff
 *
 * Petar T. Petkoff <tihomiroff@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License version 2
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>

#include "gpios.h"

#define _SerialPortDevice 	"/dev/ttyS0"
#define _BaudRate		B57600

/*---------------------------------------------------------------------------*/
int main() {
    int fd, res;
    struct termios oldtio, newtio;
    
    fd = open(_SerialPortDevice, O_RDWR | O_NOCTTY );
    if (fd < 0) { perror(_SerialPortDevice); exit(-1); }

    tcgetattr(fd, &oldtio); /* save current port settings */
      
    bzero(&newtio, sizeof(newtio));
    // baudrate, 8 bit, ignore modem, enable
    newtio.c_cflag = _BaudRate | CS8 | CLOCAL | CREAD;
    // ignore parity, xon/xoff flow
    newtio.c_iflag = IGNPAR | IXON | IXOFF;
    newtio.c_oflag = 0;
    newtio.c_lflag = 0;
    newtio.c_cc[VTIME]    = 0;
    newtio.c_cc[VMIN]     = 1;
    tcflush(fd, TCOFLUSH);
    tcsetattr(fd,TCSANOW,&newtio);
    ioctl(fd, TIOCMGET, &res);

    disable_communication(fd);

    close(fd);

    return 0;
}
/*---------------------------------------------------------------------------*/

/* end of disable_comminucation.c */
